#include "level.h"


// OK
Level::Level() : m_name("DEFAULT"), m_pigs(NULL), m_nbPigs(0), m_attracts(NULL), m_nbAttracts(0), m_bow(Bow()), m_ScrMX(64), m_ScrMY(32), m_zoom(1.), m_prevL_x(NULL), m_prevL_y(NULL), m_lenPrevL(0)
{
	for(int k=0; k<4; k++)
		m_box[k] = 0;
	for(int k=0; k<3; k++)
		m_scores[k] = 0;
	for(int k=0; k<5; k++)
		m_piafs[k] = 0;
}

// OK
Level::Level(char *name) : m_name(""), m_pigs(NULL), m_nbPigs(0), m_attracts(NULL), m_nbAttracts(0), m_ScrMX(64), m_ScrMY(32), m_zoom(1.), m_prevL_x(NULL), m_prevL_y(NULL), m_lenPrevL(0)
{
	for(int k=0; k<9; k++)
	{
		m_name[k] = name[k];
		if(name[k] == '\0') break;
	}
	
	for(int k=0; k<4; k++)
		m_box[k] = 0;
	for(int k=0; k<3; k++)
		m_scores[k] = 0;
	for(int k=0; k<5; k++)
		m_piafs[k] = 0;
	
	m_bow = Bow();
}

// OK
Level::~Level()
{
	free(m_pigs);
	free(m_attracts);
	free(m_prevL_x);
	free(m_prevL_y);
}


// OK
int Level::load(int memory)
{
	char buf[60];
	char *data=NULL;
	int index=0;
	int startLine=0;
	int section=0;
	
	// Chargement
	sprintf(buf, "\\\\%s\\%s.lvl", (memory==SD ? "crd0" : "fls0"), m_name);
	
	data = (char*) memory_load(buf);
	if(data==NULL)
		return -2;
	
	
	// Analyse
	while(data[index] != '#')
	{
		startLine = index;
		while(data[index] != '\n')
		{
			buf[index-startLine] = data[index];
			index++;
		}
		buf[index-startLine] = '\0';
		index++;
		
		if(buf[0]=='[')
		{
			section++;
		}
		else if(buf[0]!='\0' && buf[0]!='\n')
		{
			if(section==1) // BOX
			{
				int i=0;
				int k=0;
				int startN=0;
				
				while(buf[i] != '\0')
				{
					char strN[5];
					
					startN = i;
					while(buf[i] != ';' && buf[i] != '\0')
					{
						strN[i-startN] = buf[i];
						i++;
					}
					strN[i-startN] = '\0';
					i++;
					
					m_box[k] = atoi(strN);
					k++;
				}
			}
			else if(section==2) // SCORES
			{
				int i=0;
				int k=0;
				int startN=0;
				
				while(buf[i] != '\0')
				{
					char strN[7];
					
					startN = i;
					while(buf[i] != ';' && buf[i] != '\0')
					{
						strN[i-startN] = buf[i];
						i++;
					}
					strN[i-startN] = '\0';
					i++;
					
					m_scores[k] = atoi(strN);
					//DEBUG(m_scores[k]);
					k++;
				}
			}
			else if(section==3) // PIAFS
			{
				int i=0;
				int k=0;
				int startN=0;
				
				while(buf[i] != '\0')
				{
					char strN[5];
					
					startN = i;
					while(buf[i] != ';' && buf[i] != '\0')
					{
						strN[i-startN] = buf[i];
						i++;
					}
					strN[i-startN] = '\0';
					i++;
					
					m_piafs[k] = atoi(strN);
					k++;
				}
			}
			else if(section==4) // PIGS
			{
				m_nbPigs++;
				m_pigs = (Pig *)realloc(m_pigs, m_nbPigs*sizeof(Pig));
				
				TypePig type;
				int x, y;
				int orient = 0;
				
				int i=0;
				int k=0;
				int startN=0;
				
				while(buf[i] != '\0')
				{
					char strN[5];
					
					startN = i;
					while(buf[i] != ';' && buf[i] != '\0')
					{
						strN[i-startN] = buf[i];
						i++;
					}
					strN[i-startN] = '\0';
					i++;
					
					if(k==0)
					{
						switch(atoi(strN))
						{
							case 0:
								type = soldier;
								break;
							case 1:
								type = king;
								break;
							case 2:
								type = egg;
								break;
						}
					}
					else if(k==1)
						x = atoi(strN);
					else if(k==2)
						y = atoi(strN);
					else if(k==3)
						orient = atoi(strN);
					
					k++;
				}
				
				m_pigs[m_nbPigs-1] = Pig(x, y, type, orient);
			}
			else if(section==5) // ATTRACTORS
			{
				m_nbAttracts++;
				m_attracts = (Attractor *)realloc(m_attracts, m_nbAttracts*sizeof(Attractor));
				
				TypeAttract type;
				int x, y;
				int orient = 0;
				
				int i=0;
				int k=0;
				int startN=0;
				
				while(buf[i] != '\0')
				{
					char strN[5];
					
					startN = i;
					while(buf[i] != ';' && buf[i] != '\0')
					{
						strN[i-startN] = buf[i];
						i++;
					}
					strN[i-startN] = '\0';
					i++;
					
					if(k==0)
						type = (TypeAttract)atoi(strN);
					else if(k==1)
						x = atoi(strN);
					else if(k==2)
						y = atoi(strN);
					else if(k==3)
						orient = atoi(strN);
					
					k++;
				}
				
				m_attracts[m_nbAttracts-1] = Attractor(x, y, type, orient);
			}
			else if(section==6) // BOW
			{
				int x, y;
				int orient = 0;
				
				int i=0;
				int k=0;
				int startN=0;
				
				while(buf[i] != '\0')
				{
					char strN[5];
					
					startN = i;
					while(buf[i] != ';' && buf[i] != '\0')
					{
						strN[i-startN] = buf[i];
						i++;
					}
					strN[i-startN] = '\0';
					i++;
					
					if(k==0)
						x = atoi(strN);
					else if(k==1)
						y = atoi(strN);
					else if(k==2)
						orient = atoi(strN);
					
					k++;
				}
				
				m_bow = Bow(x, y, orient);
			}
		}
	}
	
	free(data);
	return 0;
}

// OK
void Level::debug(int memory) const
{
	int key=0;
	char buf[100];
	
	dclear();
	sprintf(buf, "\\\\%s\\%s.lvl", (memory==SD ? "crd0" : "fls0"), m_name);
	dtext(0, 0, buf);
	sprintf(buf, "BOX: %d;%d %d;%d", m_box[0], m_box[1], m_box[2], m_box[3]);
	dtext(0, 8, buf);
	sprintf(buf, "SCORE: %d %d %d", m_scores[0], m_scores[1], m_scores[2]);
	dtext(0, 16, buf);
	sprintf(buf, "PIAFS: %d %d %d %d %d", m_piafs[0], m_piafs[1], m_piafs[2], m_piafs[3], m_piafs[4]);
	dtext(0, 24, buf);
	sprintf(buf, "PIGS: %d", m_nbPigs);
	dtext(0, 32, buf);
	sprintf(buf, "ATTRACTORS: %d", m_nbAttracts);
	dtext(0, 40, buf);
	sprintf(buf, "BOW: x=%d y=%d", (int)m_bow.get_x(), (int)m_bow.get_y());
	dtext(0, 48, buf);
	dtext(50, 58, "PRESS [EXE]");
	dupdate();
	
	while(key!=KEY_EXE)
		key = getkey();
}


// OK
int Level::play()
{
	bool followPiaf=true, opt=false;
	char buf[50]="\0";
	int *temp=NULL;
	int count = 0;
	
	Control zoomP(KEY_PLUS), zoomM(KEY_MINUS);
	Control left(KEY_LEFT, COOLDOWN_CAMERA), right(KEY_RIGHT, COOLDOWN_CAMERA), up(KEY_UP, COOLDOWN_CAMERA), down(KEY_DOWN, COOLDOWN_CAMERA);
	Control alpha(KEY_ALPHA), F2(KEY_F2);
	
	m_zoom = 1.;
	
	while(havePiaf())
	{
		Piaf piaf(m_bow.get_x(), m_bow.get_y(), luke);
		
		// Before shooting
		while(IsKeyUp(KEY_SHIFT))
		{
			// Graphics
			dclear();
			if(followPiaf)
				piaf.get_pos(&m_ScrMX, &m_ScrMY);
			draw();
			if(opt)
				drawOpt();
			piaf.draw(m_ScrMX, m_ScrMY, m_zoom);
			dupdate();
			
			// Controls
			if(zoomM.test() && m_zoom>.25)
				m_zoom /= 2.;
			if(zoomP.test() && m_zoom<2.)
				m_zoom *= 2.;
			
			if(left.test() && m_ScrMX>m_box[0])
			{
				followPiaf = false;
				m_ScrMX--;
			}
			if(right.test() && m_ScrMX<m_box[2])
			{
				followPiaf = false;
				m_ScrMX++;
			}
			if(up.test() && m_ScrMY>m_box[1])
			{
				followPiaf = false;
				m_ScrMY--;
			}
			if(down.test() && m_ScrMY<m_box[3])
			{
				followPiaf = false;
				m_ScrMY++;
			}
			if(alpha.test())
				followPiaf = !followPiaf;
			if(F2.test())
				opt = !opt;
			if(IsKeyDown(KEY_EXIT) || IsKeyDown(KEY_F6))
				return 0;
			
			setFps(3);
		}
		
		
		// Shooting
		dclear();
		draw();
		if(opt)
			drawOpt();
		dupdate();
		
		m_bow.shoot(&piaf, m_attracts, m_nbAttracts, m_ScrMX, m_ScrMY, m_zoom);
		
		m_lenPrevL = 0;
		if(m_prevL_x)
		{
			free(m_prevL_x);
			m_prevL_x = NULL;
		}
		if(m_prevL_y)
		{
			free(m_prevL_y);
			m_prevL_y = NULL;
		}
		
		// Main loop
		while(IsKeyUp(KEY_EXIT) && !piafOut(piaf) && !piaf.isStop())
		{
			// Graphics
			dclear();
			if(followPiaf)
				piaf.get_pos(&m_ScrMX, &m_ScrMY);
			if(opt)
				drawOpt();
			draw();
			piaf.draw(m_ScrMX, m_ScrMY, m_zoom);
			dupdate();
			

			// Save el camino del piaf
			count++;
			if(count>STEP_PATH*2 && true)
			{
				count=0;
				m_lenPrevL++;
				temp = (int *)realloc(m_prevL_x, m_lenPrevL*sizeof(int));
				if(temp!=NULL)
				{
					m_prevL_x = temp;
					temp = NULL;
				}
				else
				{
					m_lenPrevL = 0;
					if(m_prevL_x)
					{
						free(m_prevL_x);
						m_prevL_x = NULL;
					}
					if(m_prevL_y)
					{
						free(m_prevL_y);
						m_prevL_y = NULL;
					}
				}
				temp = (int *)realloc(m_prevL_y, m_lenPrevL*sizeof(int));
				if(temp!=NULL)
				{
					m_prevL_y = temp;
					temp = NULL;
				}
				else
				{
					m_lenPrevL = 0;
					if(m_prevL_x)
					{
						free(m_prevL_x);
						m_prevL_x = NULL;
					}
					if(m_prevL_y)
					{
						free(m_prevL_y);
						m_prevL_y = NULL;
					}
				}
				
				if(m_prevL_x!=NULL && m_prevL_y!=NULL)
				{
					m_prevL_x[m_lenPrevL-1] = INT(piaf.get_x());
					m_prevL_y[m_lenPrevL-1] = INT(piaf.get_y());
				}
			}


			// Physical Engine
			piaf.resetAcc();
			for(int k=0; k<m_nbAttracts; k++)
				piaf.calculateAttract(m_attracts[k]);
			piaf.calculateSpd(T_INTEG);
			piaf.calculatePos(T_INTEG);
			
			for(int k=0; k<m_nbAttracts; k++)
				piaf.colliAttract(&m_attracts[k]);
			for(int k=0; k<m_nbPigs; k++)
				piaf.colliPig(&m_pigs[k]);
			
			
			// Controls
			if(zoomM.test() && m_zoom>.25)
				m_zoom /= 2.;
			if(zoomP.test() && m_zoom<2.)
				m_zoom *= 2.;
			
			if(left.test() && m_ScrMX>m_box[0])
			{
				followPiaf = false;
				m_ScrMX--;
			}
			if(right.test() && m_ScrMX<m_box[2])
			{
				followPiaf = false;
				m_ScrMX++;
			}
			if(up.test() && m_ScrMY>m_box[1])
			{
				followPiaf = false;
				m_ScrMY--;
			}
			if(down.test() && m_ScrMY<m_box[3])
			{
				followPiaf = false;
				m_ScrMY++;
			}
			if(alpha.test())
				followPiaf = !followPiaf;
			if(F2.test())
				opt = !opt;
			if((IsKeyDown(KEY_EXIT) && IsKeyDown(KEY_SHIFT)) || IsKeyDown(KEY_F6))
				return 0;
			
			setFps(3);
		}
		
		sleep_ms(500);
		popUp("Press [EXE]", KEY_EXE);
	}
	
	return 1;
}

// OK
void Level::draw() const
{
	int ScrOX = m_ScrMX*m_zoom-64;
	int ScrOY = m_ScrMY*m_zoom-32;
	
	for(int k=0; k<m_nbPigs; k++)
		m_pigs[k].draw(m_ScrMX, m_ScrMY, m_zoom);
	
	for(int k=0; k<m_nbAttracts; k++)
		m_attracts[k].draw(m_ScrMX, m_ScrMY, m_zoom);
	
	for(int k=0; k<m_lenPrevL; k++)
		dpixel(INT(m_prevL_x[k]*m_zoom)-ScrOX, INT(m_prevL_y[k]*m_zoom)-ScrOY, color_black);
	
	m_bow.draw(m_ScrMX, m_ScrMY, m_zoom);
}

// OK
void Level::drawOpt() const
{
	int ScrOX = m_ScrMX*m_zoom-64;
	int ScrOY = m_ScrMY*m_zoom-32;
	
	drect((int)(m_box[0]*m_zoom)-ScrOX, (int)(m_box[1]*m_zoom)-ScrOY, (int)(m_box[2]*m_zoom)-ScrOX, (int)(m_box[3]*m_zoom)-ScrOY, color_black);
}

// OK
bool Level::piafOut(Piaf piaf) const
{
	int x, y;
	int radius = (int)piaf.get_radius();
	
	piaf.get_pos(&x, &y);
	
	if(x-radius<m_box[0] || x+radius>m_box[2]  ||  y-radius<m_box[1] || y+radius>m_box[3])
		return true;
	return false;
}


// OK
int Level::havePiaf() const
{
	int nb=0;
	
	for(int k=0; k<5; k++)
		nb += m_piafs[k];
	
	return nb;
}

// OK
void Level::popUp(char *string) const
{
	int l = Draw_stringLength(string);
	
	drect(60-l, POPUP_Y-2, 68+l, POPUP_Y+6, color_black);
	drect(61-l, POPUP_Y-1, 67+l, POPUP_Y+5, color_white);
	Draw_printCenter(string, POPUP_Y, 64);
}

// OK
void Level::popUp(char *string, unsigned int keyRequired) const
{
	int key=0;
	int l = Draw_stringLength(string)/2;
	
	drect(60-l, POPUP_Y-2, 68+l, POPUP_Y+6, color_black);
	drect(61-l, POPUP_Y-1, 67+l, POPUP_Y+5, color_white);
	Draw_printCenter(string, POPUP_Y, 64);
	dupdate();

	while(key!=keyRequired)
		key = getkey();
}

