/****************************************************************/
/*                                                              */
/*                            Memory                            */
/*                                                              */
/*  Description: Fonctions de manipulation de la memoire        */
/*  Auteur:      LePhenixNoir                                   */
/*  Version:     3.0                                            */
/*  Date:        11.06.2014                                     */
/*  Fichier:     memory.c - Code des fonctions                  */
/*                                                              */
/****************************************************************/

#ifndef _STDIO
  #include <stdio.h>
#endif

#ifndef _STDLIB
  #include <stdlib.h>
#endif

#ifndef _STRING
  #include <string.h>
#endif

#include "memory.h"

int memory_errors = 0;

void memory_seterrors(int e)
{
  memory_errors = (e!=0);
}

void memory_error(char *from, char *func, int val)
{
  char info[20];
  if(!memory_errors) return;  

  sprintf(info,"%d",val);

  dclear();
  dtext(0, 0,   (unsigned char *)"Memory ERROR !!");
  dtext(10, 10, (unsigned char *)"FROM:");
  dtext(30, 10, (unsigned char *)from);
  dtext(10, 20, (unsigned char *)"FUNC:");
  dtext(30, 20, (unsigned char *)func);
  dtext(10, 30, (unsigned char *)"INFO:");
  dtext(30, 30, (unsigned char *)info);
  dtext(10, 40, (unsigned char *)"META:");

  switch(val)
  {
    case  1:  dtext(30, 40, (unsigned char *)"NotEnoughRAM"); break;
    case -1:  dtext(30, 40, (unsigned char *)"Nonexisting");  break;
    case -5:  dtext(30, 40, (unsigned char *)"WrongDevice");  break;
    case -8:  dtext(30, 40, (unsigned char *)"AccessDenied"); break;
    case -14: dtext(30, 40, (unsigned char *)"ReadOnly");     break;
    case -31: dtext(30, 40, (unsigned char *)"DeviceError");  break;
    case -35: dtext(30, 40, (unsigned char *)"NotEmpty");     break;
    default:  dtext(30, 40, (unsigned char *)"Other");        break;
  }
  getkey();
}

FONTCHARACTER *memory_char2font(char *adresse)
{
  FONTCHARACTER *adr;
  int i;

  adr = calloc((strlen(adresse)+1),sizeof(FONTCHARACTER));
  for(i=0;i<strlen(adresse);i++) *(adr+i) = *(adresse+i);
  return adr;
}

int memory_createfile(char *adresse, int size)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = BFile_Create(adr, BFile_File, &size);
  if(i<0) memory_error("createfile()","CreateFile()",i);

  free(adr);
  return i;
}

int memory_createdir(char *adresse)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = BFile_Create(adr, BFile_Folder, NULL);
  if(i<0) memory_error("createdir()","CreateDir.()",i);

  free(adr);
  return 1;
}

int memory_openfile(char *adresse, enum BFile_OpenMode mode)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = BFile_Open(adr, mode);
  if(i<0) memory_error("openfile()", "OpenFile()", i);

  free(adr);
  return i;
}

int memory_deletefile(char *adresse)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = BFile_Remove(adr);
  if(i<0) memory_error("deletefil.()","DeleteFil.()",i);

  free(adr);
  return i;
}

char **memory_alloc(int l)
{
  char **p = calloc(l,sizeof(char *));
  int i; for(i=0;i<l;i++) *(p+i) = calloc(20,1);
  return p;
}

void memory_free(char **p, int l)
{
  int i; for(i=0;i<l;i++) free(*(p+i));
  free(p);
}

void *memory_load(char *adresse)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int handle, x, size;
  void *p;

  if((handle=BFile_Open(adr,BFile_ReadOnly))<0)
  {
    memory_error("load()","OpenFile()",handle);
    return NULL;
  }
  size = BFile_GetFileSize(handle)+1;
  p = calloc(size,1);

  if(!p)
  {
    memory_error("load()","malloc()",1);
    BFile_Close(handle); free(adr); return NULL; 
  }
  if((x=BFile_Read(handle, p, size, 0))<0)
  {
    memory_error("load()","ReadFile()",x);
    BFile_Close(handle); free(adr); return NULL;
  }

  BFile_Close(handle);
  free(adr);
  return p;
}

// int memory_save(char *adresse, void *data, int l)
// {
//   FONTCHARACTER *adr = memory_char2font(adresse);
//   int x=0, handle;
//   int lp = l+1;

//   if(memory_exists(adresse)) x = BFile_Remove(adr);
//   if(x<0)
//   {
//     memory_error("save()","DeleteFile()",x);
//     free(adr);
//     return x;
//   }

//   x = BFile_Create(adr, BFile_File, &lp);
//   if(x<0)
//   {
//     memory_error("save()","CreateFile()",x);
//     free(adr);
//     return x;
//   }
//   handle = BFile_Open(adr, BFile_WriteOnly);
//   if(handle<0)
//   {
//     memory_error("save()","OpenFile()",handle);
//     free(adr);
//     return handle;
//   }
//   x = memory_writefile(handle, data, l);
//   if(x<0)
//   {
//     memory_error("save()","WriteFile()",x);
//     free(adr);
//     return x;
//   }
//   memory_closefile(handle);

//   free(adr);
//   return 0;
// }
