#include "mobile.h"

Mobile::Mobile() :
Object(), m_vx(0.), m_vy(0.), m_ax(0.), m_ay(0.), m_orient(0), m_mass(1.), m_oldX(-100), m_oldY(-100), m_stepStop(STEPS_STOP)
{
	
}

Mobile::Mobile(int x, int y, float mass) :
Object(x, y), m_vx(0.), m_vy(0.), m_ax(0.), m_ay(0.), m_orient(0), m_mass(mass), m_oldX(-100), m_oldY(-100), m_stepStop(STEPS_STOP)
{
	
}

Mobile::Mobile(int x, int y, float mass, color_t color) :
Object(x, y, color), m_vx(0.), m_vy(0.), m_ax(0.), m_ay(0.), m_orient(0), m_mass(mass), m_oldX(-100), m_oldY(-100), m_stepStop(STEPS_STOP)
{
	
}

Mobile::Mobile(int x, int y, float mass, int radius, color_t color) :
Object(x, y, radius, color), m_vx(0.), m_vy(0.), m_ax(0.), m_ay(0.), m_orient(0), m_mass(mass), m_oldX(-100), m_oldY(-100), m_stepStop(STEPS_STOP)
{
	
}

Mobile::~Mobile()
{
	
}




////////////////////////////////////////// PUBLIC //////////////////////////////////////////

void Mobile::calculatePos(float dT)
{
	m_x = m_x + m_vx*dT;
	m_y = m_y + m_vy*dT;
}

void Mobile::calculateSpd(float dT)
{
	m_vx = m_vx + m_ax*dT;
	m_vy = m_vy + m_ay*dT;
}

void Mobile::calculateAttract(Attractor attractor)
{
	float dist_x = (attractor.get_x() - m_x);
	float dist_y = (attractor.get_y() - m_y);
	float r2 = dist_x*dist_x + dist_y*dist_y;
	
	if(r2>(attractor.get_radiusAttract()*attractor.get_radiusAttract()) || r2<(attractor.get_radius()+m_radius)*(attractor.get_radius()+m_radius))
		return;
	
	m_ax += dist_x * attractor.get_gravity() /r2/m_mass;
	m_ay += dist_y * attractor.get_gravity() /r2/m_mass;
}

void Mobile::calculateOrient()
{
	if(ABS(m_vx)>ABS(m_vy))
	{
		if(m_vx>0)
			m_orient = 0;
		else
			m_orient = 180;
	}
	else
	{
		if(m_vy<0)
			m_orient = 90;
		else
			m_orient = 270;
	}
}


bool Mobile::colliObject(Object *object, float bounceFact)
{
	float dist_x = (object->get_x() - m_x);
	float dist_y = (object->get_y() - m_y);
	float minD = m_radius+object->get_radius();
	float r = dist_x*dist_x + dist_y*dist_y;
	
	if(r<minD*minD && m_vx*dist_x+m_vy*dist_y>0)
	{
		// Calcul de la direction de la tangente entre les deux cercles
		float dir_x = -dist_y/sqrt(r);
		float dir_y = dist_x/sqrt(r);
		// Calcul de la projection de la vitesse sur la tangente
		float proj_x = (dir_x*m_vx+dir_y*m_vy) * dir_x;
		float proj_y = (dir_x*m_vx+dir_y*m_vy) * dir_y;
		
		m_vx = (2*proj_x - m_vx)*bounceFact;
		m_vy = (2*proj_y - m_vy)*bounceFact;
		
		return true;
	}
	
	return false;
}

bool Mobile::isStop()
{
	m_stepStop--;
	if(m_stepStop <= 0)
	{
		if(ABS(m_x-m_oldX)<.05 && ABS(m_y-m_oldY)<.05)
			return true;
		m_oldX = m_x;
		m_oldY = m_y;
		m_stepStop = STEPS_STOP;
	}
	
	return false;
}



////////////////////////////////////////// GETTERS //////////////////////////////////////////

float Mobile::get_mass() const
{
	return m_mass;
}

float Mobile::get_spd() const
{
	return sqrt(m_vx*m_vx+m_vy*m_vy);
}

void Mobile::get_spd(float *vx, float *vy) const
{
	*vx = m_vx;
	*vy = m_vy;
}

void Mobile::get_acc(float *ax, float *ay) const
{
	*ax = m_ax;
	*ay = m_ay;
}



////////////////////////////////////////// SETTERS //////////////////////////////////////////

void Mobile::set_spd(float vx, float vy)
{
	m_vx = vx;
	m_vy = vy;
}

void Mobile::set_orient(int orient)
{
	m_orient = orient;
}

void Mobile::resetAcc()
{
	m_ax = 0.;
	m_ay = 0.;
}

